/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */
package com.jcg.verifycertificate;

import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import java.io.FileOutputStream;
import java.security.*;
import java.security.cert.X509Certificate;
import java.math.BigInteger;
import org.bouncycastle.jce.X509Principal;
import org.bouncycastle.x509.X509V3CertificateGenerator;
import java.util.Date;

/**
 *
 * @author omozegieaziegbe
 */
public class VerifyCertificate {

    public static void main(String[] args) {
        // Add Bouncy Castle as a security provider
        Security.addProvider(new BouncyCastleProvider());

        try {
            // Generate a key pair
            KeyPair keyPair = generateKeyPair();

            // Generate a self-signed certificate
            X509Certificate selfSignedCertificate = generateSelfSignedCertificate(keyPair);

            // Check if the certificate is self-signed
            if (isSelfSigned(selfSignedCertificate)) {
                System.out.println("The certificate is Self-signed.");
            } else {
                System.out.println("The certificate is CA-signed.");
            }

            // Save the certificate to a file (optional)
            saveCertificateToFile(selfSignedCertificate, "/Users/omozegieaziegbe/Development/self_signed_certificate.cer");

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public static boolean isSelfSigned(X509Certificate certificate) {
        try {
            certificate.verify(certificate.getPublicKey());
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    // Generate a key pair using RSA algorithm
    private static KeyPair generateKeyPair() throws NoSuchAlgorithmException {
        KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("RSA");
        keyPairGenerator.initialize(2048); // Key size
        return keyPairGenerator.generateKeyPair();
    }

    // Generate a self-signed certificate
    public static X509Certificate generateSelfSignedCertificate(KeyPair keyPair) throws GeneralSecurityException {
        X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();

        certGen.setSerialNumber(BigInteger.valueOf(System.currentTimeMillis()));
        certGen.setIssuerDN(new X509Principal("CN=javacodegeeks"));
        certGen.setSubjectDN(new X509Principal("CN=javacodegeeks"));
        certGen.setPublicKey(keyPair.getPublic());
        certGen.setNotBefore(new Date(System.currentTimeMillis()));
        certGen.setNotAfter(new Date(System.currentTimeMillis() + 365 * 24 * 60 * 60 * 1000)); // 1 year validity
        certGen.setSignatureAlgorithm("SHA256WithRSAEncryption");

        return certGen.generate(keyPair.getPrivate(), "BC");
    }

    // Save the certificate to a file
    private static void saveCertificateToFile(X509Certificate certificate, String fileName) throws Exception {
        try (FileOutputStream fos = new FileOutputStream(fileName)) {
            fos.write(certificate.getEncoded());
        }
    }

}
